package cn.iocoder.yudao.module.reward.controller.admin.reward;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.module.reward.convert.reward.RewardConvert;
import cn.iocoder.yudao.module.reward.dal.dataobject.reward.RewardDO;
import cn.iocoder.yudao.module.reward.service.reward.RewardService;
import cn.iocoder.yudao.module.reward.vo.reward.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;

@Validated
@RestController
@Api(tags = "管理后台 - 礼品管理")
@RequestMapping("/ecw/reward/")
public class RewardController {

    @Resource
    private RewardService rewardService;

    @PostMapping("/create")
    @ApiOperation("创建礼品")
    @Idempotent(timeout = 3)
    @PreAuthorize("@ss.hasPermission('ecw:giftManagement:add')")
    public CommonResult<List<Long>> create(@Valid @RequestBody RewardCreateReqVO createReqVO) {
        return success(rewardService.create(createReqVO));
    }

    @PostMapping("/update")
    @ApiOperation("更新礼品")
    @PreAuthorize("@ss.hasPermission('ecw:giftManagement:edit')")
    public CommonResult<Boolean> update(@Valid @RequestBody RewardUpdateReqVO updateReqVO) {
        rewardService.update(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除礼品")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:giftManagement:delete')")
    public CommonResult<Boolean> delete(@RequestParam("id") Long id) {
        rewardService.delete(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得礼品详情")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:giftManagement:check')")
    public CommonResult<RewardBackVO> get(@RequestParam("id") Long id) {
        RewardDO rewardDO = rewardService.get(id);
        RewardBackVO rewardBackVO = RewardConvert.INSTANCE.convert(rewardDO);
        return success(rewardBackVO);
    }

    @GetMapping("/list")
    @ApiOperation("获得礼品列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    public CommonResult<List<RewardBackVO>> getList(@RequestParam("ids") Collection<Long> ids) {
        List<RewardDO> list = rewardService.getList(ids);
        List<RewardBackVO> rewardBackVOS = RewardConvert.INSTANCE.convertList(list);
        return success(rewardBackVOS);
    }

    @PostMapping("/page")
    @ApiOperation("获得礼品分页")
    public CommonResult<PageResult<RewardBackVO>> getPage(@Valid @RequestBody RewardQueryVO query, PageVO page) {
        PageResult<RewardDO> pageResult = rewardService.getPage(query, page);
        PageResult<RewardBackVO> rewardBackVOPageResult = RewardConvert.INSTANCE.convertPage(pageResult);
        // 转换nodeId和points
        rewardBackVOPageResult.getList().forEach(rewardBackVO -> {
            List<Integer> nodeId = new ArrayList<>();
            nodeId.add(rewardBackVO.getNodeId());
            rewardBackVO.getNodeIds().add(new RewardBackVO.NodeAndPoints(nodeId, rewardBackVO.getQuantityRemain(), rewardBackVO.getPointsRequire()));
        });
        return success(rewardBackVOPageResult);
    }

    @PostMapping("/status")
    @ApiOperation("礼品状态变更")
    @PreAuthorize("@ss.hasPermission('ecw:giftManagement:close')")
    public CommonResult<Boolean> updateStatus(@Valid @RequestBody RewardStatusReqVO statusVO) {
        rewardService.updateStatus(statusVO);
        return success(true);
    }

    @PostMapping("/copy/{id}")
    @ApiOperation("复制礼品")
    @Idempotent(timeout = 3)
    @PreAuthorize("@ss.hasPermission('ecw:giftManagement:copy')")
    public CommonResult<Long> copyReward(@NotNull @PathVariable Long id) {
        return success(rewardService.copyReward(id));
    }

    @PostMapping("/delay")
    @ApiOperation("礼品延期")
    @PreAuthorize("@ss.hasPermission('ecw:giftManagement:postpone')")
    public CommonResult<Boolean> delayReward(@Valid @RequestBody RewardDelayReqVO delayVO) {
        rewardService.delayReward(delayVO);
        return success(true);
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出礼品 Excel")
    @PreAuthorize("@ss.hasPermission('reward::export')")
    @OperateLog(type = EXPORT)
    public void exportExcel(@Valid RewardQueryVO query,
                            HttpServletResponse response) throws IOException {
        List<RewardDO> list = rewardService.getList(query);
        // 导出 Excel
        List<RewardBackVO> datas = RewardConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "礼品.xls", "数据", RewardBackVO.class, datas);
    }
}
