package cn.iocoder.yudao.module.reward.util;

import cn.iocoder.yudao.framework.redis.helper.RedisHelper;
import cn.iocoder.yudao.module.reward.dal.dataobject.redeem.RewardRedeemDO;
import cn.iocoder.yudao.module.reward.dal.dataobject.reward.RewardDO;
import cn.iocoder.yudao.module.reward.dal.mysql.redeem.RewardRedeemMapper;
import cn.iocoder.yudao.module.reward.dal.mysql.reward.RewardMapper;
import com.alibaba.excel.util.DateUtils;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.infra.enums.ErrorCodeConstants.GET_LOCK_FAILED;

/**
 * 根据规则生成各种code
 */
@Component
public class RewardGenCodeUtils {
    @Resource
    private RewardMapper rewardMapper;
    @Resource
    private RedisHelper redisHelper;
    @Resource
    private RedissonClient redissonClient;
    @Resource
    private RewardRedeemMapper rewardRedeemMapper;

    /**
     * 生成礼品ID
     */
    public String generateRewardCode() {
        // 礼品ID命名规则：G+年+5位序列号，例如G2400005
        String key = "rewardCode:max:number";
        StringBuilder code = new StringBuilder();
        code.append("G");
        code.append(DateUtils.format(new Date(), "yy"));
        Long codeNum;
        if (redisHelper.hasKey(key)) {
            codeNum = redisHelper.incrBy(key, 1);
        } else {
            RLock lock = redissonClient.getLock("next:reward:code:lock");
            try {
                boolean lockSuccess = lock.tryLock(2, 2, TimeUnit.SECONDS);
                if (!lockSuccess) {
                    throw exception(GET_LOCK_FAILED);
                }
                String currentMaxRewardCode = rewardMapper.getCurrentMaxRewardCode();
                if (currentMaxRewardCode == null){
                    codeNum = 1L;
                } else {
                    codeNum = Long.parseLong(currentMaxRewardCode.substring(3)) + 1;
                }
                redisHelper.set(key, String.valueOf(codeNum),10, TimeUnit.SECONDS);
            } catch (InterruptedException e) {
                throw exception(GET_LOCK_FAILED);
            } finally {
                lock.unlock();
            }
        }
        // 获得5位序列号，不足位前面补0
        codeNum = codeNum % 100000;
        code.append(String.format("%05d", codeNum));
        return code.toString();
    }
    /**
     * 生成兑换记录编号
     */
    public String generateRedemptionNumber() {

        String key = "redemptionNumber:max:number";
        // EC+年月+5位序列号，例如EC240900005
        StringBuilder code = new StringBuilder();
        code.append("EC");
        String currentYearMonth = DateUtils.format(new Date(), "yyMM");
        code.append(currentYearMonth);
        Long codeNum;
        if (redisHelper.hasKey(key)) {
            codeNum = redisHelper.incrBy(key, 1);
        } else {
            RLock lock = redissonClient.getLock("next:redemptionNumber:code:lock");
            try {
                boolean lockSuccess = lock.tryLock(2, 2, TimeUnit.SECONDS);
                if (!lockSuccess) {
                    throw exception(GET_LOCK_FAILED);
                }
                String currentMaxRedemptionNumber = rewardRedeemMapper.getCurrentMaxRedemptionNumber();
                if (currentMaxRedemptionNumber == null){
                    codeNum = 1L;
                } else {
                    codeNum = Long.parseLong(currentMaxRedemptionNumber.substring(6)) + 1;
                }
                redisHelper.set(key, String.valueOf(codeNum),10, TimeUnit.SECONDS);
            } catch (InterruptedException e) {
                throw exception(GET_LOCK_FAILED);
            } finally {
                lock.unlock();
            }
        }
        // 获得5位序列号，不足位前面补0
        codeNum = codeNum % 100000;
        code.append(String.format("%05d", codeNum));
        return code.toString();
    }
    public void historyRewardCodeFlush() {
        long start = 1L;
        redisHelper.delete("rewardCode:max:number");
        List<RewardDO> rewardDOS = rewardMapper.selectList();
        for (RewardDO rewardDO : rewardDOS) {
            StringBuilder code = new StringBuilder();
            code.append("G");
            code.append(DateUtils.format(new Date(), "yy"));
            code.append(String.format("%05d", start));
            rewardDO.setCode(code.toString());
            rewardMapper.updateById(rewardDO);
            start += 1;
        }
    }
    public void redemptionNumberFlush() {
        long start = 1L;
        redisHelper.delete("redemptionNumber:max:number");
        List<RewardRedeemDO> rewardRedeemDOList = rewardRedeemMapper.selectList();
        for (RewardRedeemDO rewardRedeemDO : rewardRedeemDOList) {
            StringBuilder code = new StringBuilder();
            code.append("EC");
            String currentYearMonth = DateUtils.format(new Date(), "yyMM");
            code.append(currentYearMonth);
            code.append(String.format("%05d", start));
            rewardRedeemDO.setRedemptionNumber(code.toString());
            rewardRedeemMapper.updateById(rewardRedeemDO);
            start += 1;
        }
    }
}
